@@ -115,6 +115,9 @@ def execute_mysql_write_query(
     # >ACTION(S)                                              #
     # ##########################################################
     # CREATE DB CURSOR
+
+    log.info('starting execute_mysql_write_query')
+
     try:
         cursor = dbConn.cursor(MySQLdb.cursors.DictCursor)
     except Exception, e:
@@ -138,6 +141,8 @@ def execute_mysql_write_query(
         cursor.close()
     except Exception, e:
         log.warning('could not close the db cursor ' + str(e) + '\n')
+
+    log.info('finished execute_mysql_write_query')
     return None
 
 
@@ -193,49 +198,85 @@ def execute_mysql_read_query(
 ## CREATED : 20121023
 
 def convert_dictionary_to_mysql_table(
-    dbConn,
-    log,
-    dictionary,
-    dbTableName,
-    uniqueKeyList,
-    ):
+        dbConn,
+        log,
+        dictionary,
+        dbTableName,
+        uniqueKeyList=[],
+        createHelperTables=False):
     """ Convert a python dictionary into a mysql table
 
-            ****Key Arguments:****
-                - ``dictionary`` -- python dictionary
-                - ``dbConn`` -- the db connection
-                - ``dbTableName`` -- name of the table you wish to add the data to (or create if it does not exist)
-                - ``uniqueKeyList`` - a tuple of lowcase, no space mysql column names that need to be the primary key
+    **Key Arguments:**
+        - ``log`` -- logger
+        - ``dictionary`` -- python dictionary
+        - ``dbConn`` -- the db connection
+        - ``dbTableName`` -- name of the table you wish to add the data to (or create if it does not exist)
+        - ``uniqueKeyList`` - a lists column names that need combined to create the primary key
+        - ``createHelperTables`` -- create some helper tables with the main table, detailing original keywords etc
 
-            **Return:**
-                - ``None`` """
+    **Return:**
+        - ``None`` """
 
     # # >IMPORTS ##
-    import MySQLdb
+    import MySQLdb as mdb
     import re
     import yaml
     import time
-    import dryxPython.commonutils
-    import ordereddict as c  # REMOVE WHEN PYTHON 2.7 INSTALLED ON PSDB
-    # import collections as c
-    # # >SETTINGS ##
+    from dryxPython import commonutils as dcu
+    # import ordereddict as c  # REMOVE WHEN PYTHON 2.7 INSTALLED ON PSDB
+    import collections as c
+
+    log.info('starting convert_dictionary_to_mysql_table')
+
+    ## TEST THE ARGUMENTS
+    if str(type(dbConn).__name__) != "Connection":
+        message = 'Please use a valid MySQL DB connection.'
+        log.critical(message)
+        raise TypeError(message)
+
+    if not isinstance(dictionary, dict):
+        message = 'Please make sure "dictionary" argument is a dict type.'
+        log.critical(message)
+        raise TypeError(message)
+
+    if not isinstance(uniqueKeyList, list):
+        message = 'Please make sure "uniqueKeyList" is a list'
+        log.critical(message)
+        raise TypeError(message)
+
+    for i in uniqueKeyList:
+        if i not in dictionary.keys():
+            message = 'Please make sure values in "uniqueKeyList" are present in the "dictionary" you are tring to convert'
+            log.critical(message)
+            raise ValueError(message)
+
+    for k, v in dictionary.iteritems():
+        if isinstance(v, list) and len(v) != 2:
+            message = 'Please make sure the list values in "dictionary" 2 items in length'
+            log.critical("%s: in %s we have a %s (%s)" % (message,k,v,type(v)))
+            raise ValueError(message)
+        if not (isinstance(v[0], str) or isinstance(v[0], int) or isinstance(v[0], bool) or isinstance(v[0], float) or v[0] == None):
+            message = 'Please make sure values in "dictionary" are of an appropriate value to add to the database, must be str, float, int or bool'
+            log.critical("%s: in %s we have a %s (%s)" % (message,k,v,type(v)))
+            raise ValueError(message)
+
+    if not isinstance(createHelperTables, bool):
+        message = 'Please make sure "createHelperTables" is a True or False'
+        log.critical(message)
+        raise TypeError(message)
+
     qCreateColumnPreamble = \
         "IF NOT EXISTS( (SELECT * FROM information_schema.COLUMNS WHERE TABLE_SCHEMA=DATABASE() AND COLUMN_NAME='my_additional_column' AND TABLE_NAME='my_table_name') ) THEN"
     qCreateColumnPostamble = \
-        'PRIMARY KEY (`primaryId`)) ENGINE=MyISAM AUTO_INCREMENT=0 DEFAULT CHARSET=latin1;\nSET character_set_client = @saved_cs_client;\n'
+        'PRIMARY KEY (`primaryId`)) ENGINE=MyISAM AUTO_INCREMENT=0 DEFAULT CHARSET=latin1;'
     # MySQL DOES NOT LIKE COMPOUNDED QUERIES FROM MYSQLDB - CREATE LIST OF QUERIES INSTEAD
     qCreateTableCommandList = []
-    qCreateTableCommandList.extend(['SET @saved_cs_client = @@character_set_client'])
-    qCreateTableCommandList.extend(['SET character_set_client = utf8'])
-    qCreateTableCommandList.extend(['SET sql_notes = 0'])
     qCreateTableCommandList.extend(["""CREATE TABLE IF NOT EXISTS `""" + dbTableName
                                    + """`
                                             (`primaryId` bigint(20) NOT NULL AUTO_INCREMENT COMMENT 'An internal counter',
                                             PRIMARY KEY (`primaryId`))
                                             ENGINE=MyISAM AUTO_INCREMENT=0 DEFAULT CHARSET=latin1"""
                                    ])
-    qCreateTableCommandList.extend(['SET @@character_set_client = @saved_cs_client'])
-    qCreateTableCommandList.extend(['SET sql_notes = 1'])
     reFeedParserClass = re.compile('FeedParserDict')
     reDatetime = re.compile('^[0-9]{4}-[0-9]{2}-[0-9]{2}T')
     reTypeTime = re.compile('struct_time')
@@ -243,9 +284,7 @@ def convert_dictionary_to_mysql_table(
     formattedKey = ''
     formattedKeyList = []
     myValues = []
-    # ##########################################################
-    # >ACTIONS                                                #
-    # ##########################################################
+
     # CREATE THE TABLE IF IT DOES NOT EXIST YET
     try:
         # log.debug("creating the "+dbTableName+" table if it does not exist")
@@ -266,36 +305,40 @@ def convert_dictionary_to_mysql_table(
         log.critical('could not create the table ' + dbTableName + ' ' + str(e) + '\n')
         sys.exit(e)
     # ADD EXTRA COLUMNS TO THE DICTIONARY
-    dictionary['dateCreated'] = str(commonutils.get_now_sql_datetime())
-    dictionary['dateLastModified'] = str(commonutils.get_now_sql_datetime())
-    dictionary['dateLastRead'] = str(commonutils.get_now_sql_datetime())
+    dictionary['dateCreated'] = [str(dcu.get_now_sql_datetime()),"date row was created"]
+    dictionary['dateLastModified'] = [str(dcu.get_now_sql_datetime()),"date row was modified"]
+    dictionary['dateLastRead'] = [str(dcu.get_now_sql_datetime()),"date row was last read"]
     # ITERATE THROUGH THE DICTIONARY AND GENERATE THE A TABLE COLUMN WITH THE NAME OF THE KEY, IF IT DOES NOT EXIST
     count = len(dictionary)
     i = 1
     for (key, value) in dictionary.items():
-        if value is None:
+        if value[0] is None:
             del dictionary[key]
     # SORT THE DICTIONARY BY KEY
     odictionary = c.OrderedDict(sorted(dictionary.items()))
     for (key, value) in odictionary.iteritems():
-        formattedKey = key
+
+        formattedKey = key.replace(" ","_").replace("-","_")
         # DEC A KEYWORD IN MYSQL - NEED TO CHANGE BEFORE INGEST
-        if formattedKey == 'dec':
-            formattedKey = 'decl'
+        if formattedKey == "dec":
+            formattedKey = "decl"
+        if formattedKey == "DEC":
+            formattedKey = "DECL"
+
         formattedKeyList.extend([formattedKey])
         if len(key) > 0:
             # CONVERT LIST AND FEEDPARSER VALUES TO YAML (SO I CAN PASS IT AS A STRING TO MYSQL)
-            if type(value) == list or reFeedParserClass.search(str(type(value))):
-                value = yaml.dump(value)
-                value = str(value)
+            if type(value[0]) == list or reFeedParserClass.search(str(type(value[0]))):
+                value[0] = yaml.dump(value[0])
+                value[0] = str(value[0])
             # REMOVE CHARACTERS THAT COLLIDE WITH MYSQL
-            if type(value) == str or type(value) == unicode:
-                value = value.replace('"', """'""")
+            # if type(value[0]) == str or type(value[0]) == unicode:
+            #     value[0] = value[0].replace('"', """'""")
             # JOIN THE VALUES TOGETHER IN A LIST - EASIER TO GENERATE THE MYSQL COMMAND LATER
-            if type(value) == unicode:
-                myValues.extend(['%s' % value.strip()])
+            if type(value[0]) == unicode:
+                myValues.extend(['%s' % value[0].strip()])
             else:
-                myValues.extend(['%s' % (value, )])
+                myValues.extend(['%s' % (value[0], )])
             # CHECK IF COLUMN EXISTS YET
             colExists = \
                 "SELECT *\
@@ -315,33 +358,33 @@ def convert_dictionary_to_mysql_table(
                 log.error('something went wrong' + str(e) + '\n')
             # IF COLUMN DOESN'T EXIT - GENERATE IT
             if len(rows) == 0:
-                qCreateColumn = 'ALTER TABLE ' + dbTableName + ' ADD ' + formattedKey
-                if reDatetime.search(str(value)):
+                qCreateColumn = """ALTER TABLE %s ADD %s""" % (dbTableName,formattedKey)
+                if reDatetime.search(str(value[0])):
                     # log.debug('Ok - a datetime string was found')
                     qCreateColumn += ' datetime DEFAULT NULL'
                 elif formattedKey == 'updated_parsed' or formattedKey == 'published_parsed' or formattedKey \
                     == 'feedName' or formattedKey == 'title':
-                    qCreateColumn += ' varchar(200) DEFAULT NULL'
-                elif (type(value) == str or type(value) == unicode) and len(value) < 30:
-                    qCreateColumn += ' varchar(450) DEFAULT NULL'
-                elif (type(value) == str or type(value) == unicode) and len(value) >= 30 and len(value) < 80:
-                    qCreateColumn += ' varchar(450) DEFAULT NULL'
-                elif type(value) == str or type(value) == unicode:
-                    columnLength = 450 + len(value) * 2
+                    qCreateColumn += ' varchar(100) DEFAULT NULL'
+                elif (type(value[0]) == str or type(value[0]) == unicode) and len(value[0]) < 30:
+                    qCreateColumn += ' varchar(100) DEFAULT NULL'
+                elif (type(value[0]) == str or type(value[0]) == unicode) and len(value[0]) >= 30 and len(value[0]) < 80:
+                    qCreateColumn += ' varchar(100) DEFAULT NULL'
+                elif type(value[0]) == str or type(value[0]) == unicode:
+                    columnLength = 450 + len(value[0]) * 2
                     qCreateColumn += ' varchar(' + str(columnLength) + ') DEFAULT NULL'
-                elif type(value) == int and abs(value) <= 9:
+                elif type(value[0]) == int and abs(value[0]) <= 9:
                     qCreateColumn += ' tinyint DEFAULT NULL'
-                elif type(value) == int:
+                elif type(value[0]) == int:
                     qCreateColumn += ' int DEFAULT NULL'
-                elif type(value) == float or type(value) == long:
+                elif type(value[0]) == float or type(value[0]) == long:
                     qCreateColumn += ' double DEFAULT NULL'
-                elif type(value) == bool:
+                elif type(value[0]) == bool:
                     qCreateColumn += ' tinyint DEFAULT NULL'
-                elif type(value) == list:
+                elif type(value[0]) == list:
                     qCreateColumn += ' varchar(1024) DEFAULT NULL'
                 else:
                     log.debug('Do not know what format to add this key in MySQL - removing from dictionary: %s, %s'
-                              % (key, type(value)))
+                              % (key, type(value[0])))
                     formattedKeyList.pop()
                     myValues.pop()
                     qCreateColumn = None
@@ -361,33 +404,41 @@ def convert_dictionary_to_mysql_table(
                         log.debug('qCreateColumn: %s' % (qCreateColumn, ))
                         log.error('could not create the ' + formattedKey + ' column in the ' + dbTableName
                                   + ' table -- ' + str(e) + '\n')
+
     # # GENERATE THE INDEX NAME - THEN CREATE INDEX IF IT DOES NOT YET EXIST
-    # CHECK IF THE UNIQUE KEY IS A LIST (COULD BE STRING)
-    if type(uniqueKeyList) is list:
-        indexName = uniqueKeyList[0]
+    if len(uniqueKeyList):
+        for i in range(len(uniqueKeyList)):
+            uniqueKeyList[i] = uniqueKeyList[i].replace(" ","_").replace("-","_")
+            if uniqueKeyList[i] == "dec":
+                uniqueKeyList[i] = "decl"
+            if uniqueKeyList[i] == "DEC":
+                uniqueKeyList[i] = "DECL"
+
+        indexName = uniqueKeyList[0].replace(" ","_").replace("-","_")
         for i in range(len(uniqueKeyList) - 1):
             indexName += '_' + uniqueKeyList[i + 1]
-    else:
-        indexName = uniqueKeyList
-    indexName = commonutils.make_lowercase_nospace(indexName)
-    rows = execute_mysql_read_query(
-        """SELECT COUNT(*) FROM INFORMATION_SCHEMA.STATISTICS WHERE TABLE_SCHEMA = DATABASE() AND
-                                TABLE_NAME = '"""
-             + dbTableName + """' AND INDEX_NAME = '""" + indexName + """'""",
-        dbConn,
-        log,
-        )
-    exists = rows[0]['COUNT(*)']
-    if exists == 0:
-        if type(uniqueKeyList) is list:
-            uniqueKeyList = ','.join(uniqueKeyList)
-        addUniqueKey = 'ALTER TABLE ' + dbTableName + ' ADD unique ' + indexName + """ (""" + uniqueKeyList + ')'
-        # log.debug('HERE IS THE COMMAND:'+addUniqueKey)
-        execute_mysql_write_query(
-            addUniqueKey,
+
+        indexName = dcu.make_lowercase_nospace(indexName)
+        rows = execute_mysql_read_query(
+            """SELECT COUNT(*) FROM INFORMATION_SCHEMA.STATISTICS WHERE TABLE_SCHEMA = DATABASE() AND
+                                    TABLE_NAME = '"""
+                 + dbTableName + """' AND INDEX_NAME = '""" + indexName + """'""",
             dbConn,
             log,
             )
+        exists = rows[0]['COUNT(*)']
+        log.debug('uniqueKeyList: %s' % (uniqueKeyList,))
+        if exists == 0:
+            if type(uniqueKeyList) is list:
+                uniqueKeyList = ','.join(uniqueKeyList)
+
+            addUniqueKey = 'ALTER TABLE ' + dbTableName + ' ADD unique ' + indexName + """ (""" + uniqueKeyList + ')'
+            # log.debug('HERE IS THE COMMAND:'+addUniqueKey)
+            execute_mysql_write_query(
+                addUniqueKey,
+                dbConn,
+                log,
+                )
     # GENERATE THE INSERT COMMAND - IGNORE DUPLICATE ENTRIES
     myKeys = ','.join(formattedKeyList)
     myValues = '" ,"'.join(myValues)
@@ -401,10 +452,10 @@ def convert_dictionary_to_mysql_table(
     myValues = myValues.replace('!!python/unicode:', '')
     myValues = myValues.replace('!!python/unicode', '')
     # log.debug(myValues+" ------ POSTSTRIP")
-    addValue = """INSERT IGNORE INTO """ + dbTableName + """ (""" + myKeys + """) VALUES (\"""" + myValues + """\")"""
+    addValue = """INSERT INTO """ + dbTableName + """ (""" + myKeys + """) VALUES (\"""" + myValues + """\")"""
     # log.debug(addValue)
     try:
-        log.debug('adding new data to the %s table; query: %s' % (dbTableName, addValue))
+        # log.debug('adding new data to the %s table; query: %s' % (dbTableName, addValue))
         execute_mysql_write_query(
             addValue,
             dbConn,
@@ -412,6 +463,9 @@ def convert_dictionary_to_mysql_table(
             )
     except Exception, e:
         log.error("could not add new data added to the table '" + dbTableName + "' : " + str(e) + '\n')
+
+    log.info('finished convert_dictionary_to_mysql_table')
+
     return None
 
 
@@ -516,17 +570,17 @@ def add_column_to_db_table(
 ## CREATED : 20121102
 
 def add_HTMIds_to_mysql_tables(
-    raColName,
-    declColName,
-    tableName,
-    dbConn,
-    log,
-    ):
+        raColName,
+        declColName,
+        tableName,
+        dbConn,
+        log,
+        primaryIdColumnName="primaryId"):
     """ Calculate and append HTMId info to a mysql db table containing ra and dec columns
 
     ****Key Arguments:****
-        - ``ra`` -- ra in sexegesimal
-        - ``decl`` -- dec in sexegesimal
+        - ``raColName`` -- ra in sexegesimal
+        - ``declColName`` -- dec in sexegesimal
         - ``tableName`` -- name of table to add htmid info to
         - ``dbConn`` -- database hosting the above table
         - ``log`` -- logger
@@ -535,9 +589,40 @@ def add_HTMIds_to_mysql_tables(
         - ``None`` """
 
     # # > IMPORTS ##
-    import utils as u
+    import MySQLdb as ms
     import dryxPython.mysql as m
+    from dryxPython.kws import utils as u
     # # >SETTINGS ##
+
+    ## TEST TABLE EXIST
+    sqlQuery = """show tables"""
+    rows =  execute_mysql_read_query(
+        sqlQuery=sqlQuery,
+        dbConn=dbConn,
+        log=log
+    )
+    tableList = []
+    for row in rows:
+        tableList.extend(row.values())
+    if tableName not in tableList:
+        message = "The %s table does not exist in the database" % (tableName,)
+        log.critical(message)
+        raise IOError(message)
+
+    ## TEST COLUMN EXISTS
+    cursor = dbConn.cursor(ms.cursors.DictCursor)
+    sqlQuery = """SELECT * FROM %s LIMIT 1""" % (tableName,)
+    cursor.execute(sqlQuery)
+    rows = cursor.fetchall()
+    desc = cursor.description
+    existingColumns = []
+    for i in range(len(desc)):
+        existingColumns.append(desc[i][0])
+    if (raColName not in existingColumns) or (declColName not in existingColumns):
+        message = 'Please make sure you have got the naes of the RA and DEC columns correct'
+        log.critical(message)
+        raise IOError(message)
+
     # >ACTION(S)   ###
     htmCols = {
         'htm16ID': 'BIGINT(20)',
@@ -546,16 +631,17 @@ def add_HTMIds_to_mysql_tables(
         'cy': 'DOUBLE',
         'cz': 'DOUBLE',
         }
+
     # CHECK IF COLUMNS EXISTS YET - IF NOT CREATE FROM
     for key in htmCols.keys():
         try:
             log.debug('attempting to check and generate the HTMId columns for the %s db table' % (tableName, ))
             colExists = \
                 """SELECT *
-                                    FROM information_schema.COLUMNS
-                                    WHERE TABLE_SCHEMA=DATABASE()
-                                    AND COLUMN_NAME='%s'
-                                    AND TABLE_NAME='%s'""" \
+                    FROM information_schema.COLUMNS
+                    WHERE TABLE_SCHEMA=DATABASE()
+                    AND COLUMN_NAME='%s'
+                    AND TABLE_NAME='%s'""" \
                 % (key, tableName)
             colExists = m.execute_mysql_read_query(
                 colExists,
@@ -574,7 +660,7 @@ def add_HTMIds_to_mysql_tables(
                           % (tableName, str(e)))
             return -1
     # SELECT THE ROWS WHERE THE HTMIds ARE NOT SET
-    sqlQuery = 'SELECT primaryId, ' + raColName + ', ' + declColName + ' from ' + tableName + ' where htm16ID is NULL'
+    sqlQuery = """SELECT %s, %s, %s from %s where htm16ID is NULL""" % (primaryIdColumnName,raColName, declColName,tableName)
     rows = m.execute_mysql_read_query(
         sqlQuery,
         dbConn,
@@ -582,41 +668,45 @@ def add_HTMIds_to_mysql_tables(
         )
     # NOW GENERATE THE HTMLIds FOR THESE ROWS
     for row in rows:
-        (thisRa, thisDec) = (float(row[raColName]), float(row[declColName]))
-        htm16ID = u.htmID(
-            thisRa,
-            thisDec,
-            16,
-            )
-        htm20ID = u.htmID(
-            thisRa,
-            thisDec,
-            20,
-            )
-        (cx, cy, cz) = u.calculate_cartesians(thisRa, thisDec)
-        sqlQuery = \
-            """UPDATE %s SET htm16ID=%s, htm20ID=%s,cx=%s,cy=%s,cz=%s
-                                                where primaryId = '%s'""" \
-            % (
-            tableName,
-            htm16ID,
-            htm20ID,
-            cx,
-            cy,
-            cz,
-            row['primaryId'],
-            )
-        try:
-            log.debug('attempting to update the HTMIds for new objects in the %s db table' % (tableName, ))
-            m.execute_mysql_write_query(
-                sqlQuery,
-                dbConn,
-                log,
+        if row[raColName] is None or row[declColName] is None:
+            continue
+        else:
+            (thisRa, thisDec) = (float(row[raColName]), float(row[declColName]))
+            htm16ID = u.htmID(
+                thisRa,
+                thisDec,
+                16,
                 )
-        except Exception, e:
-            log.critical('could not update the HTMIds for new objects in the %s db table - failed with this error: %s '
-                         % (tableName, str(e)))
-            return -1
+            htm20ID = u.htmID(
+                thisRa,
+                thisDec,
+                20,
+                )
+            (cx, cy, cz) = u.calculate_cartesians(thisRa, thisDec)
+            sqlQuery = \
+                """UPDATE %s SET htm16ID=%s, htm20ID=%s,cx=%s,cy=%s,cz=%s
+                                                    where %s = '%s'""" \
+                % (
+                tableName,
+                htm16ID,
+                htm20ID,
+                cx,
+                cy,
+                cz,
+                primaryIdColumnName,
+                row[primaryIdColumnName],
+                )
+            try:
+                log.debug('attempting to update the HTMIds for new objects in the %s db table' % (tableName, ))
+                m.execute_mysql_write_query(
+                    sqlQuery,
+                    dbConn,
+                    log,
+                    )
+            except Exception, e:
+                log.critical('could not update the HTMIds for new objects in the %s db table - failed with this error: %s '
+                             % (tableName, str(e)))
+                return -1
     return None
 
 
